Package: TDbasedUFE
Type: Package
Title: Tensor Decomposition Based Unsupervised Feature Extraction
Version: 1.12.0
Authors@R: 
    person("Y-h.", "Taguchi", ,
    "tag@granular.com", role = c("aut", "cre"),
    comment = c(ORCID  = "0000-0003-0867-8986"))
Description: 
    This is a comprehensive package to perform
    Tensor decomposition based unsupervised feature extraction.
    It can perform unsupervised feature extraction.
    It uses tensor decomposition.
    It is  applicable to gene expression, DNA methylation, and 
    histone modification etc.
    It can perform multiomics analysis.
    It is also potentially applicable to single cell omics data sets.
biocViews: GeneExpression, FeatureExtraction, MethylationArray,
        SingleCell
License: GPL-3
Encoding: UTF-8
URL: https://github.com/tagtag/TDbasedUFE
BugReports: https://github.com/tagtag/TDbasedUFE/issues
Imports: GenomicRanges, rTensor, readr, methods, MOFAdata, tximport,
        tximportData, graphics, stats, utils, shiny
RoxygenNote: 7.2.3
Roxygen: list(markdown = TRUE)
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/TDbasedUFE
git_branch: RELEASE_3_23
git_last_commit: 1763056
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:43:38 UTC; biocbuild
Author: Y-h. Taguchi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0867-8986>)
Maintainer: Y-h. Taguchi <tag@granular.com>
Built: R 4.6.0; ; 2026-04-29 05:03:31 UTC; unix

# Generated by roxygen2: do not edit by hand

export(PrepareSummarizedExperimentTensor)
export(PrepareSummarizedExperimentTensorSquare)
export(computeHosvd)
export(computeHosvdSqure)
export(convertSquare)
export(selectFeature)
export(selectFeatureSquare)
export(selectSingularValueVectorLarge)
export(selectSingularValueVectorSmall)
export(tableFeatures)
export(tableFeaturesSquare)
import(MOFAdata)
import(methods)
import(readr)
import(tximport)
import(tximportData)
importFrom(GenomicRanges,GRanges)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(methods,new)
importFrom(rTensor,as.tensor)
importFrom(rTensor,hosvd)
importFrom(shiny,actionButton)
importFrom(shiny,fluidPage)
importFrom(shiny,mainPanel)
importFrom(shiny,observeEvent)
importFrom(shiny,plotOutput)
importFrom(shiny,renderPlot)
importFrom(shiny,runApp)
importFrom(shiny,shinyApp)
importFrom(shiny,sidebarLayout)
importFrom(shiny,sidebarPanel)
importFrom(shiny,stopApp)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(utils,menu)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# TDbasedUFE

<!-- badges: start -->
<!-- badges: end -->

The goal of TDbasedUFE is to …

## Installation

You can install the latest release of TDbasedUFE from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFE@v0.1.0")
```

You can install the development version of TDbasedUFE from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFE",build_vignettes = TRUE)
```

or if it does not work, try

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFE")
```

# TDbasedUFE

**Tensor decomposition based unsupervised feature extraction**

## Manuscript

The theoretical back ground of this package was described in the
following book:

***“Unsupervised Feature Extraction Applied to Bioinformatics: A PCA
Based and TD Based Approach”*** by Y-h. Taguchi

Please cite the book as:

``` latex
@book{Taguchi2020,
  doi = {10.1007/978-3-030-22456-1},
  url = {https://doi.org/10.1007/978-3-030-22456-1},
  year = {2020},
  publisher = {Springer International Publishing},
  author = {Y-H. Taguchi},
  title = {Unsupervised Feature Extraction Applied to Bioinformatics}
}
```

[\[DOI\]](https://doi.org/10.1007/978-3-030-22456-1)

## Vignettes

How to use it

vignette(“QuickStart”)

For more theoretical background

vignette(“TDbasedUFE”)

#' Title
#' Generate feature values formatted as a tensor format from Squared matrix
#' @param sample Sample names
#' @param feature Feature id names
#' @param value Squared Feature values
#' @param featureRange Genomic coordinate attributed to feature id (if any)
#' @param sampleData Sample property (labels etc)
#'
#' @return A tensor including sample names, feature values,
#' associated with featureRange and sample properties
#' @export
#'
#' @examples
#' omics1 <- matrix(runif(100),10)
#' dimnames(omics1) <- list(seq_len(10),seq_len(10))
#' omics2 <- matrix(runif(100),10)
#' dimnames(omics2) <- dimnames(omics1)
#' Multi <- list(omics1,omics2)
#'Z <- PrepareSummarizedExperimentTensorSquare(
#'    sample=matrix(colnames(omics1),1),
#'    feature=list(omics1=rownames(omics1),
#'    omics2=rownames(omics2)),
#'    value=convertSquare(Multi),
#'    sampleData=list(NA))
PrepareSummarizedExperimentTensorSquare <- function(sample=list(NULL),
    feature,value,featureRange=GRanges(NULL),
    sampleData=list(NULL))
{
    new("SummarizedExperimentTensorSquare",
        sample=sample,feature=feature,value=value,
        featureRange=featureRange,sampleData=sampleData)
}

test_that("multiplication works", {
require(GenomicRanges)
require(rTensor)
library("readr")
library("tximport")
library("tximportData")
dir <- system.file("extdata", package="tximportData")
samples <- read.table(file.path(dir,"samples.txt"), header=TRUE)
samples$condition <- factor(rep(c("A","B"),each=3))
rownames(samples) <- samples$run
samples[,c("pop","center","run","condition")]
files <- file.path(dir,"salmon", samples$run, "quant.sf.gz")
names(files) <- samples$run
tx2gene <- read_csv(file.path(dir, "tx2gene.gencode.v27.csv"))
txi <- tximport(files, type="salmon", tx2gene=tx2gene)
txi[seq_len(3)] <-
    lapply(txi[seq_len(3)],function(x){dim(x);x[seq_len(10000),]})


Z <- PrepareSummarizedExperimentTensor(matrix(samples$sample,c(3,2)),
    rownames(txi$abundance),array(txi$counts,c(dim(txi$counts)[1],3,2)))
HOSVD <- computeHosvd(Z)
input_all <- selectSingularValueVectorSmall(HOSVD,input_all=c(1,2))
index <- selectFeature(HOSVD,input_all)


Z <- PrepareSummarizedExperimentTensor(matrix(samples$sample,c(6,1)),
rownames(txi$abundance),array(txi$counts,c(dim(txi$counts)[1],6)))
HOSVD <- computeHosvd(Z)
cond <- list(0,rep(c("A","B"),each=3))
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=2)

#MultiOmics
require(MOFAdata)
data("CLL_data")
data("CLL_covariates")
Z <- PrepareSummarizedExperimentTensorSquare(
    sample=matrix(colnames(CLL_data$Drugs),1),
    feature=list(Drugs=rownames(CLL_data$Drugs),
    Methylation=rownames(CLL_data$Methylation),
    mRNA=rownames(CLL_data$mRNA),Mutations=rownames(CLL_data$Mutations)),
    value=convertSquare(CLL_data),sampleData=list(CLL_covariates[,1]))
HOSVD <- computeHosvdSqure(Z)
cond <- list(attr(Z,"sampleData")[[1]],attr(Z,"sampleData")[[1]],seq_len(4))
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(8,1))
index <- selectFeatureSquare(HOSVD,input_all,CLL_data,
    de=c(0.3,0.03,0.1,0.1),interact=FALSE)
})

#' Title
#' Select singular value vectors from HOSVD
#' @param HOSVD output from HOSVD
#' @param input_all if ist is no null, no interactive mode is
#'  activated but provided values are used.
#'
#' @return Selected singular value vector IDs
#' @export
#'
#' @examples
#'Z <- PrepareSummarizedExperimentTensor(
#'sample=matrix(as.character(seq_len(6)),c(3,2)),
#' feature=as.character(seq_len(10)),
#'  value=array(runif(10*3*2),c(10,3,2)))
#'  HOSVD <- computeHosvd(Z)
#' input_all <- selectSingularValueVectorSmall(HOSVD,input_all=c(1,1))
selectSingularValueVectorSmall <- function(HOSVD,input_all=NULL){
    interact<-FALSE
    if (is.null(input_all))interact <- TRUE
    for (i in 2:length(HOSVD$U))
    {
        j<-1
        if (interact)
        {
            ui <- fluidPage(
                sidebarLayout(
                    sidebarPanel(
                        actionButton(inputId="action", label="Next"),
                        actionButton(inputId="prev",  label="Prev"), 
                        actionButton(inputId="select", label="Select")),
                    mainPanel(
                        plotOutput("plot")
                    )
                )
            )
            server <- function(input, output){
                observeEvent(input$action, {
                    if (j<dim(HOSVD$U[[i]])[2]) j<<-j+1
                })
                observeEvent(input$prev, {
                    if (j!=1){j<<-j-1}
                })  
                observeEvent(input$select, {
                ; stopApp()
                })  
                output$plot <- renderPlot({
                    input$action
                    input$prev
                    if(length(unique(sign(HOSVD$U[[i]][,j])))==1)
                    {
                        RANGE <- range(c(0,range(HOSVD$U[[i]][,j])))
                    } else
                    {
                        RANGE<-range(HOSVD$U[[i]][,j])
                    }
                    plot(HOSVD$U[[i]][,j],type="h",ylim=RANGE,main=j)
                    abline(0,0,col=2,lty=2)
                })
            }
            app<- shinyApp(ui, server)
            runApp(app)
            input_all <- c(input_all,j)
        } else
        {
            if(length(unique(sign(HOSVD$U[[i]][,input_all[i-1]])))==1)
            {
                RANGE <- range(c(0,range(HOSVD$U[[i]][,input_all[i-1]])))
            } else
            {
                RANGE<-range(HOSVD$U[[i]][,input_all[i-1]])
            }
            plot(HOSVD$U[[i]][,input_all[i-1]],type="h",
                ylim=RANGE,main=input_all[i-1])
            abline(0,0,col=2,lty=2)
        }
    }
    return(input_all)
}

#' Title
#' Generate feature values formatted as a tensor format
#' @param sample Sample names
#' @param feature Feature id names
#' @param value Feature values
#' @param featureRange Genomic coordinate attributed to feature id (if any)
#' @param sampleData Sample property (labels etc)
#'
#'
#' @return A tensor including sample names, feature id,
#'  feature values, associated with featureRange and sample properties
#' @export
#'
#' @examples
#' require(GenomicRanges)
#'Z <- PrepareSummarizedExperimentTensor(
#'sample=matrix(as.character(seq_len(6)),c(3,2)),
#' feature=as.character(seq_len(10)),
#'  value=array(runif(10*3*2),c(10,3,2)))
PrepareSummarizedExperimentTensor <- function(sample,feature,value,
    featureRange=GRanges(NULL),sampleData=list(NULL))
{
    new("SummarizedExperimentTensor",sample=sample,
        feature=feature,value=value,
        featureRange=featureRange,sampleData=sampleData)
}

#' Title
#'  Select features (for tensor generated from squared matrix)
#' @param HOSVD output from HOSVD applied to tensor generated 
#' from squared matrix
#' @param input_all Selected singular value vector IDs
#' @param Multi Multiomics data
#' @param de Initial value for optimization of standard deviation
#' @param p0 Threshold P-value
#' @param breaks The number of bins
#' @param interact if interact mode or not
#'
#' @return List that includes selected features and computed P-value
#' @export
#'
#' @examples
#' omics1 <- matrix(runif(100000),ncol=10)
#' dimnames(omics1) <- list(seq_len(10000),seq_len(10))
#' omics2 <- matrix(runif(100000),ncol=10)
#' dimnames(omics2) <- dimnames(omics1)
#' Multi <- list(omics1,omics2)
#' Z <- PrepareSummarizedExperimentTensorSquare(
#' sample=matrix(colnames(omics1),1),
#' feature=list(omics1=rownames(omics1),
#' omics2=rownames(omics2)),
#' value=convertSquare(Multi),
#' sampleData=list(NA))
#' HOSVD <- computeHosvdSqure(Z)
#' cond <- list(0,rep(seq_len(2),each=5),c("A","B"))
#' input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(1,1))
#' index <- selectFeatureSquare(HOSVD,input_all,Multi,de=c(0.1,0.1),
#' interact=FALSE)
selectFeatureSquare<-function(HOSVD,input_all,Multi,
    de=rep(1e-4,dim(HOSVD$U[[3]])[2]),p0=0.01,breaks=100,interact=TRUE)
{
    th <- function(sd,breaks,p0){
        P2 <- pchisq((u/sd)^2,1,lower.tail=FALSE)
        hc<- hist(1-P2,breaks=breaks,plot=FALSE)
        return(sd(hc$count[seq_len(sum(hc$breaks
        <1-min(P2[p.adjust(P2,"BH")>p0])))]))
    }
    index_all <- rep(list(NA),dim(HOSVD$U[[3]])[2])
    Multi <- lapply(Multi,function(x){x[is.na(x)]<-0;return(x)})
    for (i in seq_len(dim(HOSVD$U[[3]])[2]))
    {
        u<-scale(Multi[[i]] %*% HOSVD$U[[2]][,input_all[1]]) 
        #changed [[1]] ->[[2]] in the above
        sd <- optim(de[i],function(x){th(x,breaks,p0)},
            control=list(warn.1d.NelderMead=FALSE))$par
        sd1 <- seq(0.1*sd,2*sd,by=0.1*sd)
        th0 <- apply(matrix(sd1,ncol=1),1,function(x){th(x,breaks,p0)})
        P2 <- pchisq((u/sd)^2,1,lower.tail=FALSE)
        par(mfrow=c(1,2))
        plot(sd1,th0,type="o")
        arrows(sd,max(th0),sd,min(th0),col=2)
        hist(1-P2,breaks=breaks)
        par(mfrow=c(1,1))
        if (interact)
        {
        readline("Press Enter to proceed:")
        }
        index <- p.adjust(P2,"BH")<p0
        index_all[[i]] <- list(index=index,p.value=P2)
    }
    return(index_all)
}

#' Title
#' Select features
#' @param HOSVD output from HOSVD
#' @param input_all  Selected singular value IDs
#' @param de Initial value for optimization of standard deviation
#' @param p0 Threshold P-value
#' @param breaks The number of bins
#'
#' @return List that includes selected features and computed P-value
#' @export
#'
#' @examples
#' set.seed(2)
#' require(rTensor)
#' HOSVD <- hosvd(as.tensor(array(runif(10000*3*3),c(10000,3,3))),c(10,3,3))
#' input_all <- c(2,2)
#' index <- selectFeature(HOSVD,input_all,de=0.01,p0=0.01)
selectFeature<-function(HOSVD,input_all,de=1e-4,p0=0.01,breaks=100)
{
    th <- function(sd,breaks,p0){
    P2 <- pchisq((u/sd)^2,1,lower.tail=FALSE)
    hc<- hist(1-P2,breaks=breaks,plot=FALSE)
        return(sd(hc$count[seq_len(sum(hc$breaks
        <1-min(P2[p.adjust(P2,"BH")>p0])))]))
    }
    column<-seq_len(dim(attr(HOSVD$Z,"data"))[1])
    for (i in 2:length(HOSVD$U)){
        column<- cbind(column,input_all[i-1])
    }
    u<-HOSVD$U[[1]][,which.max(abs(attr(HOSVD$Z,"data")[column]))]
    sd <- optim(de,function(x){th(x,breaks,p0)},
        control=list(warn.1d.NelderMead=FALSE))$par
    sd1 <- seq(0.1*sd,2*sd,by=0.1*sd)
    th0 <- apply(matrix(sd1,ncol=1),1,function(x){th(x,breaks,p0)})
    P2 <- pchisq((u/sd)^2,1,lower.tail=FALSE)
    par(mfrow=c(1,2))
    plot(sd1,th0,type="o")
    arrows(sd,max(th0),sd,min(th0),col=2)
    hist(1-P2,breaks=breaks)
    par(mfrow=c(1,1))
    index <- p.adjust(P2,"BH")<p0
    return(list(index=index,p.value=P2))
}

#' Title
#'Select singular value vectors from HOSVD (boxplot version)
#' @param HOSVD output from HOSVD
#' @param cond Labels to select singular value vector number
#' @param input_all if list is not null, no interactive mode is
#' activated but provided values are used.
#'
#' @return Selected singular value vector IDs
#' @export
#'
#' @examples
#'Z <- PrepareSummarizedExperimentTensor(
#'sample=matrix(as.character(seq_len(6)),c(3,2)),
#' feature=as.character(seq_len(10)),
#'  value=array(runif(10*3*2),c(10,3,2)))
#' HOSVD <- computeHosvd(Z)
#'  cond <- list(0,c("A","B","C"),c("A","B"))
#'  input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(1,1))
selectSingularValueVectorLarge <- function(HOSVD,cond,input_all=NULL){
    interact<-FALSE
    if (is.null(input_all))interact <- TRUE
    for (i in 2:length(HOSVD$U))
    {
        j<-1
        if (interact)
        {
            ui <- fluidPage(
                sidebarLayout(
                    sidebarPanel(
                        actionButton(inputId="action", label="Next"),
                        actionButton(inputId="prev",  label="Prev"), 
                        actionButton(inputId="select", label="Select")),
                    mainPanel(
                        plotOutput("plot")
                    )
                )
            )
            server <- function(input, output){
                observeEvent(input$action, {
                    if (j<dim(HOSVD$U[[i]])[2]) j<<-j+1
                })
                observeEvent(input$prev, {
                    if (j!=1){j<<-j-1}
                })  
                observeEvent(input$select, {
                     stopApp()
                })  
                output$plot <- renderPlot({
                    input$action
                    input$prev
                    par(mfrow=c(length(cond),1))
                    par(mai=c(0.3,0.2,0.2,0.2))
                    boxplot(HOSVD$U[[i]][,j]~cond[[i]],main=j)
                    abline(0,0,col=2,lty=2)
                    par(mfrow=c(1,1))
                })
            }
            app<- shinyApp(ui, server)
            runApp(app)
            input_all <- c(input_all,j)
        } else {
            boxplot(HOSVD$U[[i]][,input_all[i-1]]~cond[[i]],main=input_all[i-1])
            abline(0,0,col=2,lty=2)
        }
    }
    return(input_all)
}

#' Generate squared tensor from multiomics data
#'
#'
#' @param Multi A list that include multiomics data
#'
#' @return A tensor computed from multiomics data
#' @export
#'
#' @examples
#' omics1 <- matrix(runif(100),10)
#' dimnames(omics1) <- list(seq_len(10),seq_len(10))
#' omics2 <- matrix(runif(100),10)
#' dimnames(omics2) <- dimnames(omics1)
#' Multi <- list(omics1,omics2)
#' Z <- convertSquare(Multi)
convertSquare<-function(Multi){
    Multi <- lapply(Multi,function(x){x[is.na(x)]<-0;return(x)})
    Multi <- lapply(Multi,function(x){t(x)%*%x})
    Multi <- array(unlist(Multi),
        unlist(c(lapply(Multi,dim)[1],length(Multi))))
    return(Multi)
}

#' Title
#' Compute higher order singular value decomposition from the
#'  tensor generated from squared matrix
#' @param Z A tensor including sample names, feature values,
#'  associated with featureRange and sample properties
#' @param dims dimensions to be computed by HOSVD
#' @param scale  If value is scaled
#'
#' @return List that includes output from HOSVD
#' @export
#'
#' @examples
#' omics1 <- matrix(runif(100),10)
#' dimnames(omics1) <- list(seq_len(10),seq_len(10))
#' omics2 <- matrix(runif(100),10)
#' dimnames(omics2) <- dimnames(omics1)
#' Multi <- list(omics1,omics2)
#' Z <- PrepareSummarizedExperimentTensorSquare(
#'             sample=matrix(colnames(omics1),1),
#'             feature=list(omics1=rownames(omics1),
#'             omics2=rownames(omics2)),
#'             value=convertSquare(Multi),
#'             sampleData=list(NA))
#'             HOSVD <- computeHosvdSqure(Z)
computeHosvdSqure <-function(Z,dims=unlist(lapply(dim(attr(Z,"value")),
    function(x){min(10,x)})),scale=TRUE)
{
    if (scale)
    {
        mean <- apply(attr(Z,"value"),length(dim(attr(Z,"value"))),mean)
        attr(Z,"value") <- aperm(aperm(attr(Z,"value"),
            c(length(dim(attr(Z,"value"))),
            c(seq_len(length(dim(attr(Z,"value")))-1))))/mean,
            c(c(2:length(dim(attr(Z,"value"))),1)))
    }
    hosvd(as.tensor(attr(Z,"value")),dims)
}

#' Title
#' Show selected features as Table
#' @param Z Tensor of features
#' @param index List that includes selected features and P-values
#'
#' @return Table list of selected features
#' @export
#'
#' @examples
#' set.seed(2)
#' require(rTensor)
#' HOSVD <- hosvd(as.tensor(array(runif(10000*3*3),c(10000,3,3))),c(10,3,3))
#' input_all <- c(2,2)
#' index <- selectFeature(HOSVD,input_all,de=0.01,p0=0.01)
#' index$index[seq_len(100)] <- TRUE
#' Z <- PrepareSummarizedExperimentTensor(
#' sample=matrix(as.character(seq_len(9)),c(3,3)),
#'         feature=as.character(seq_len(10000)),
#'         value=array(runif(10000*3*3),c(10,3,3)))
#' head(tableFeatures(Z,index))
tableFeatures <- function(Z,index){
    TABLE <- data.frame(attr(Z,"feature")[index$index],
        index$p.value[index$index],
        p.adjust(index$p.value,"BH")[index$index])
    colnames(TABLE) <- c("Feature","p value","adjusted p value")
    TABLE <- TABLE[order(TABLE[,2]),]
    return(TABLE)
}

#' Title
#' Compute higher order singular value decomposition
#' @param Z array that includes omics data
#' @param dims dimensions to be computed by HOSVD
#' @param scale If value is scaled
#'
#' @return List that includes output from HOSVD
#' @export
#'
#' @examples
#'Z <- PrepareSummarizedExperimentTensor(
#'sample=matrix(as.character(seq_len(6)),c(3,2)),
#'feature=as.character(seq_len(10)),
#'value=array(runif(10*3*2),c(10,3,2)))
#'HOSVD <- computeHosvd(Z)
computeHosvd <-function(Z,dims=c(10,dim(attr(Z,"value"))[-1]),scale=TRUE)
{
    if (scale)
    {
        hosvd(as.tensor(apply(attr(Z,"value"),
        c(seq_len(length(dims)))[-1],scale)),dims)
    } else {
        hosvd(as.tensor(attr(Z,"value")),dims)
    }
}

#'
#'
#' @slot sample Sample names
#' @slot feature Feature id names
#' @slot value Feature values
#' @slot featureRange  Genomic coordinate attributed to feature id (if any)
#' @slot sampleData   Sample property (labels etc)
setClass("SummarizedExperimentTensor",slot=list(sample="array",
    feature="character",value="array",
    featureRange="GRanges",sampleData="list"))
#' @slot sample Sample names
#' @slot feature Feature id names
#' @slot value Feature values
#' @slot featureRange Genomic coordinate attributed to feature id (if any)
#' @slot sampleData  Sample property (labels etc)
setClass("SummarizedExperimentTensorSquare",slot=list(sample="array",
    feature="list",value="array",
    featureRange="GRanges",sampleData="list"))

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import methods
#' @import readr
#' @import MOFAdata
#' @import tximport
#' @import tximportData
#' @importFrom shiny fluidPage sidebarLayout  sidebarPanel actionButton 
#' mainPanel plotOutput observeEvent runApp shinyApp stopApp renderPlot
#' @importFrom methods new
#' @importFrom GenomicRanges GRanges
#' @importFrom rTensor hosvd as.tensor
#' @importFrom stats pchisq p.adjust optim
#' @importFrom graphics hist par arrows boxplot abline
#' @importFrom utils menu
#'
## usethis namespace: end
NULL

---
title: "TDbasedUFE"
author:
- name: Y-h. Taguchi
affiliation:  Department of Physics, Chuo University, Tokyo 112-8551, Japan
email: tag@granular.com
output: 
    BiocStyle::html_document:
        toc: true
bibliography: references.bib 
vignette: >
    %\VignetteIndexEntry{TDbasedUFE}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---


```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

```{r setup}
library(TDbasedUFE)
```

# Installation

``` {r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("TDbasedUFE")
```

# Introduction

TDbasedUFE is a R package that intends to perform various analyses using
Tensor decomposition based unsupervised feature extraction (TDbasedUFE) 
[@Taguchi2020]
as much as possible in an user friendly manner. 
Although in the future, I am planning to implement all the features possible 
by TDbasedUFE in this R package, at the moment, it includes only basic 
features.

# What are tensor and tensor decomposition?

Although [tensor]
(https://bioconductor.org/packages/release/bioc/vignettes/DelayedTensor/inst/doc/DelayedTensor_1.html#12_What_is_Tensor) 
and [tensor decomposition]
(https://bioconductor.org/packages/release/bioc/vignettes/DelayedTensor/inst/doc/DelayedTensor_1.html#13_What_is_Tensor_Decomposition)
were fully described in vignettes of 
DelayedTensor[@DelayedTensor], we also briefly describe these.

Tensor is a natural extension of matrix that has only rows and columns, whereas
tensor has more number of "rows" (or "columns"). For example, three mode tensor,
$x_{ijk} \in \mathbb{R}^{N \times M \times K}$, has three suffix,
$i \in \mathbb{N}, j \in \mathbb{N}$, and $k \in \mathbb{N}$, each of which 
ranges $[1, N], [1, K]$ and $[1,M]$. 
This representation fitted with that of genomic datasets, e. g., $x_{ijk}$ can
represent gene expression of $i$th gene of $k$th tissue of $j$th subject
(person). If the measurements are performed over multiple conditions, e.g.,
different time points, we can add additional sufix that corresponds to time.
Alternatively, we can also make use of tensor to store multiomics data if we 
can add additional suffix that represent which of multiomics data is stored
in the elements of tensor. 
Since the experimental design has become more and more complicated, tensor
is an ideal data structure to store modern genomic observations.

Although tensor can store very complicated structure well, it does not always
mean that it has ability to help us to understand these complicated structure. 
In order to decompose tensor, tensor decomposition was developed. There are
various way to compose tensor. Usually, tensor decomposition (TD) is to
decompose tensor into product of some of vectors and tensors. 

The most popular TD is CP decomposition, which can decompose tensor 
into product  some of vectors. For example,
$x_{ijk} \in \mathbb{R}^{N \times M \times K}$ can
be decomposed into the product sum of three vectors, $\mathbb{R}^N,
\mathbb{R}^M$, and $\mathbb{R}^K$. 

![Tensor Decomposition](./fig1.jpg)

which can be mathematically expressed as
$$
x_{ijk} = \sum_\ell \lambda_\ell u_{\ell i} u_{\ell j} u_{\ell k}
$$
Alternatively, more advanced way, tensor train decomposition,  
can decomposes the tensor to product of a mixture of matrix and tensor as
$$
x_{ijk} = \sum_{\ell_1} \sum_{\ell_2} 
R_{\ell_1 i} R_{\ell_1 \ell_2 j} R_{\ell_2 k}
$$
However, in this specific study, we employed more primitive way of
decomposition, Tucker decomposition, as
$$
x_{ijk} = \sum_{\ell_1} \sum_{\ell_2} \sum_{\ell_3}
G(\ell_1 \ell_2 \ell_3) u_{\ell_1 i} u_{\ell_2 j} u_{\ell_3 k}
$$
where $G \in \mathbb{R}^{N \times M \times K}$ is core tensor 
that evaluate the weight of  product 
$  u_{\ell_1 i} u_{\ell_2 j} u_{\ell_3 k}$ and
$u_{\ell_1 i} \in \mathbb{R}^{N \times N},
u_{\ell_2 j} \in \mathbb{R}^{M \times M}, 
u_{\ell_3 k} \in \mathbb{R}^{K \times K}$ 
are singular value matrices and are orthogonal matrices. 

Although there are no uniqueness of Tucker decomposition, we specifically
employed higher order singular value decomposition (HOSVD) [@Taguchi2020] to get
Tucker decomposition.
The reason why we employed Tucker decomposition was fully described in 
my book[@Taguchi2020].



## Tensor decomposition based unsupervised feature extraction.

![Tensor decomposition based unsupervised feature extraction](./fig2.jpg)

Here we briefly describe how we can make use of HOSVD for the feature 
selection. Suppose $x_{ijk}$ represents the expression of gene $i$ of the
$k$th tissue of the $j$th subject (person) and the task is to select genes 
whose expression is distinct between patients and healthy controls only at one 
specific tissue. In order that, we need to select  $u_{\ell_2 j}$ attributed 
to $j$th subject which is distinct between healthy control and patients 
(see (1)) and  $u_{\ell_3 k}$ attributed to $k$th tissue is $k$th tissue
specific (see (2)). Then we next identify which $G(\ell_1 \ell_2 \ell_3)$ 
has the largest absolute value with fixed $\ell_2$ and $\ell_3$ that
correspond to distinction between patient and healthy control and tissue 
specificity, respectively (see (3)). Finally, with assuming that 
$u_{\ell_1 i}$ obeys Gaussian distribution (null hypothesis), $P$-values 
are attributed to $i$th gene.
$P$-values are corrected with considering multiple comparison corrections
(specifically, we employed Benjamini Hochberg method[@Taguchi2020]) and 
genes associated with adjusted P-values less than 0.01 are usually
selected (see (4) and (5)).

## Optimization of standard deviation

Although TD based unsupervised FE was proposed five years ago, it was 
successfully applied to wide range of genomic science, together with the 
proto type method, principal component analysis (PCA) based unsupervised
FE proposed ten years ago, we recently recognize that the optimization of 
the standard deviation (SD) in Gaussian  distribution employed as the null
hypothesis drastically improved the performance[@Taguchi2022a] 
[@Taguchi2022] [@Roy2022].
How we can optimize SD and why the optimization of SD can improve the
performance is as follows. Previously, we computed SD of 
$u_{\ell_1 i}$, $\sigma_{\ell_1}$ as
$$
\sigma_{\ell_1} = \sqrt{\frac{1}{N} \sum_i \left ( u_{\ell_1 i} - 
\langle u_{\ell_1i} \rangle\right)^2}
$$
$$
\langle u_{\ell_1 i} \rangle = \frac{1}{N} \sum_i u_{\ell_1 i}.
$$
However, $\sigma_{\ell_1}$ has tendency to be over estimated since some
of $i$th does not follow Gaussian distribution and has too large 
absolute values.
In order to exclude those $i$ that does not follow Gaussian, we tried 
to minimize SD of $h_n$ which is the frequency of $P_i$ computed 
with assuming that $u_{\ell_1 i}$ obeys Gaussian and falling into 
$n$th bin of histogram $P_i$
$$
\sigma_h = \sqrt{ \frac{1}{N_h} \sum_n \left 
( h_n - \langle h_n \rangle \right)^2}
$$
where $N_h$ is the number of bins of the histogram of $P_i$ and 
$$
\langle h_n \rangle = \frac{1}{N_h} \sum_n h_n
$$
with excluding some of $P_i$ which is supposed to be too small
to assume that obeys Gaussian (Null hypothesis).  This is because
$h_n$ should be constant if all of $P_i$ follows Gaussian (Null hypothesis).  



![Left: $\sigma_h$ as a function of $\sigma_{\ell_1}$. 
Right: Histogram, $h_n$, of $1-P_i$ being computed 
with the optimized $\sigma_{\ell_1}$.  ](./fig3.jpg)


The above plot is output of examples of function selectFeature in TDbasedUFE
package and represents the dependence of $\sigma_h$ upon $\sigma_{\ell_1}$ 
as well as the resulting histogram $h_n$ of $1-P_i$ when $u_{\ell_1 i}$ is 
fully generated from Gaussian. The smallest $\sigma_h$ correctly corresponds 
to the true $\sigma_{\ell_1} (=0.01)$ and $h_n$ is almost independent of $n$ 
as expected. 

Thus it is expected that $\sigma_h$ will take the smallest value when we 
correctly exclude $i$s whose $P_i$ does not obey Gaussian (Null hypothesis).
The following is the step to optimized $\sigma_{\ell_1}$.

1. Prepare initial $\sigma_{\ell_1}$ and define threshold adjusted $P$-value,
$P_0$.

1. Attribute $P_i$s to $i$s with assuming Gaussian distribution of 
$u_{\ell_1 i}$ using $\sigma_{\ell_1}$.

1. Attribute adjusted $P_i$s to $i$s with using Benjamini-Hochberg criterion.

1. Exclude $i$s associated with adjusted $P_i$ less than $P_0$.

1. Compute histogram of $1-P_i$ and $\sigma_h$.

1. Modify $\sigma_{\ell_1}$ until $\sigma_h$ is minimized.

1. Finally, we identify excluded $i$s  as the selected features because 
these $i$s do not obey Gaussian (Null hypothesis)  


# Multiomics data analysis

When we have multiomics data set composed of $K$ omics data measured on
common $M$ samples as
$$
x_{i_k j k} \in \mathbb{R}^{N_k \times M \times K}
$$
where $N_k$ is the number of features in $k$th omics data, we can make 
use of a tensor which is a bundle of linear kanel[@Taguchi2022b]. 
In this case, we can generate liner kernel of $k$th omics data as
$$
x_{jj'k} = \sum_{i_k} x_{i_kjk}x_{i_kj'k} \in \mathbb{R}^{M \times M \times K}
$$
to which HOSVD was applied and we get
$$
x_{jj'k} = \sum_{\ell_1} \sum_{\ell_2} \sum_{\ell_3} G(\ell_1 \ell_2 \ell_3)
u_{\ell_1 j} u_{\ell_2 j'} u_{\ell_3 k} 
$$
where $\ell_1$ and $\ell_2$ are exchangeable, since $x_{jj'k} = x_{j'jk}$.
Singular values attributed to features can be retrieved as
$$
u_{\ell i_k} = \sum_j u_{\ell j} x_{i_kj} 
$$
Then the same procedure was applied to $u_{\ell i}$ in order to select 
features.

![Multiomics analysis](./fig15.jpg)

# Conclusions

In this vignettes, we explain how TD based unsupervised FE can work
with optimized SD. In the other vignettes, we introduce how it works 
well in the real examples.

```{r}
sessionInfo()
```

---
title: "QuickStart"
author:
- name: Y-h. Taguchi
affiliation:  Department of Physics, Chuo University, Tokyo 112-8551, Japan
email: tag@granular.com
output: 
    BiocStyle::html_document:
    toc: true
bibliography: references.bib 
vignette: >
    %\VignetteIndexEntry{QuickStart}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    crop = NULL,
    comment = "#>"
)
```

# Installation

```{r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("TDbasedUFE")
```

# Introduction

```{r setup}
library(TDbasedUFE)
```

Here I introduce how we can use TDbasedUFE to process real data.

# Data Analyses

## Gene expression.

Here is how we can process real data. First we prepare the data set from
tximportdata package

```{r}
require(GenomicRanges)
require(rTensor)
library("readr")
library("tximport")
library("tximportData")
dir <- system.file("extdata", package="tximportData")
samples <- read.table(file.path(dir,"samples.txt"), header=TRUE)
samples$condition <- factor(rep(c("A","B"),each=3))
rownames(samples) <- samples$run
samples[,c("pop","center","run","condition")]
files <- file.path(dir,"salmon", samples$run, "quant.sf.gz")
names(files) <- samples$run
tx2gene <- read_csv(file.path(dir, "tx2gene.gencode.v27.csv"))
txi <- tximport(files, type="salmon", tx2gene=tx2gene)
```

As can be seen on the above, this data set is composed of six samples, which 
are divided into two classes, each of which includes three out of six samples.
The number of features is 58288

```{r}
dim(txi$counts)
```

which is too many to execute small desktops, we reduce number of features to
10,000

```{r}
txi[seq_len(3)] <-
    lapply(txi[seq_len(3)],
    function(x){dim(x);x[seq_len(10000),]})
```

Then we reformat count data, txi\$counts, as a tensor, $Z$, whose dimension is
$10000 \times 3 \times 2$ and HOSVD was applied to $Z$ to get tensor 
decomposition using function computeHosvd.

```{r}
Z <- PrepareSummarizedExperimentTensor(matrix(samples$sample,c(3,2)),
    rownames(txi$abundance),array(txi$counts,c(dim(txi$counts)[1],3,2)))
dim(attr(Z,"value"))
HOSVD <- computeHosvd(Z)
```

Here 3 and 2 stand for the number of samples in each class and the number of
classes, respectively. HOSVD includes output from HOSVD. Next, we need to decide 
which singular value vectors are used for the download analysis interactively. 
Since vignettes has no ability to store the output from interactive output from 
R, you have to input here as
```{r}
input_all <- selectSingularValueVectorSmall(HOSVD,input_all= c(1,2)) #batch mode
```
in batch mode.  Then you get the above graphic. In actual usage you can
activate interactive mode as
```
input_all <- selectSingularValueVectorSmall(HOSVD)
````
In interactive mode, you can see "Next", "Prev" and "Select" radio buttons by
which you can select singular value vectors one by one interactively. 
Now 1 and 2 is entered in input_all, since these correspond to constant among 
three samples (AAA or BBB) and distinction between A and B, respectively. 

Then we try to select features.
```{r}
index <- selectFeature(HOSVD,input_all)
```
We get the above graphic. The left one represents the dependence of "flatness"
of histogram of $P$-values computed with assuming the null hypothesis. More or
less it can select smallest value. Thus it is successful. The right one 
represents the histogram of 1-$P$-values. Peak at right end corresponds 
to genes selected (The peak at the left end does not have any meaning since
they corresponds to $P \simeq 1$).
Then we try to see top ranked features

```{r}
head(tableFeatures(Z,index))
```

These are associated with $P=0$. Thus, successful.

## Multiomics data analysis

Next we try to see how we can make use of TDbasedUFE to perform multiomics
analyses. In order that, we prepare data set using MOFAdata package as follows.

```{r}
require(MOFAdata)
data("CLL_data")
data("CLL_covariates")
help(CLL_data)
```

CLL_data is composed of four omics data,

-   Drugs: viability values in response to 310 different drugs and concentrations
-   Methylation: methylation M-values for the 4248 most variable CpG sites
-   mRNA: normalized expression values for the 5000 most variable genes
-   Mutations: Mutation status for 69 selected genes

each of which was converted to squared matrix (i. e., liner
kernel[@Taguchi2022b])and they are bundle as one tensor using
PrepareSummarizedExperimentTensorSquare function, to which HOSVD is
applied as follows.

```{r}
Z <- PrepareSummarizedExperimentTensorSquare(
    sample=matrix(colnames(CLL_data$Drugs),1),
    feature=list(Drugs=rownames(CLL_data$Drugs),
    Methylation=rownames(CLL_data$Methylation),
    mRNA=rownames(CLL_data$mRNA),Mutations=rownames(CLL_data$Mutations)),
    value=convertSquare(CLL_data),sampleData=list(CLL_covariates[,1]))
HOSVD <- computeHosvdSqure(Z)
```

CLL_covariate is labeling information, among which we employed the distinction
between male (m) and female (f).

```{r}
table(CLL_covariates[,1])
cond <- list(attr(Z,"sampleData")[[1]],attr(Z,"sampleData")[[1]],seq_len(4))
```

Since a tensor is a bundle of liner kernel, the first two singular value 
vectors are dedicated to samples and the third (last) one is dedicated to four
omics classes (Drugs, Methylation, mRNA and mutations). In order to select
which singular value vectors are employed, we execute selectFeatureSquare 
function in batch mode as follows
```{r}
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(8,1))
```
In actual usage you can activate interactive mode as
```
input_all <- selectSingularValueVectorLarge(HOSVD,cond)
```
and can select the eight singular value vetor for samples that represents
distinction between male and female and the first singular value vectors 
to four omics categories that represents the commoness between four omics 
data (i.e., constant values for four omics data).

Now we come to the stage to select features. Since there are four features,
we need to optimize SD for each of them. Try to execute selectFeatureSquare 
in a batch mode as follows.
```{r}
index <- selectFeatureSquare(HOSVD,input_all,CLL_data,
    de=c(0.3,0.03,0.1,0.1),interact=FALSE) #for batch mode
```
In actual usage, you can activate interctive mode
```
index <- selectFeatureSquare(HOSVD,input_all,CLL_data,
        de=c(0.3,0.03,0.1,0.1))
```
to see these plots one by one if you hope.
In the above, de=c(0.3,0.03,0.1,0.1) represents initial SD for the optimization
of SD for each of four omics. There might be need for some trial & errors to 
get good initial values. Every time you type enter in an interactive mode, 
you can see one of four plots for four omics data. 

Now the output includes four lists each of which corresponds to one of four
omics data. Each list is composed of two vectors whose length is equivalent to
the number of features in each of omics data. Two vectors, index and p.value,
stores logical vector that shows if individual features are selected and raw
$P$-values.

In order to see selected features for all four omics data, tableFeatureSquare
function must be repeated four times as

```{r}
for (id in seq_len(4)){print(head(tableFeaturesSquare(Z,index,id)))}
```

# Conclusion

In this vignettes, we briefly introduce how we can make use of TDbasedUFE to
perform gene expression analysis and multiomics analysis. I am glad if you can
make use of it as well for your own research.

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableFeatures.R
\name{tableFeatures}
\alias{tableFeatures}
\title{Title
Show selected features as Table}
\usage{
tableFeatures(Z, index)
}
\arguments{
\item{Z}{Tensor of features}

\item{index}{List that includes selected features and P-values}
}
\value{
Table list of selected features
}
\description{
Title
Show selected features as Table
}
\examples{
set.seed(2)
require(rTensor)
HOSVD <- hosvd(as.tensor(array(runif(10000*3*3),c(10000,3,3))),c(10,3,3))
input_all <- c(2,2)
index <- selectFeature(HOSVD,input_all,de=0.01,p0=0.01)
index$index[seq_len(100)] <- TRUE
Z <- PrepareSummarizedExperimentTensor(
sample=matrix(as.character(seq_len(9)),c(3,3)),
        feature=as.character(seq_len(10000)),
        value=array(runif(10000*3*3),c(10,3,3)))
head(tableFeatures(Z,index))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectSingularValueVectorLarge.R
\name{selectSingularValueVectorLarge}
\alias{selectSingularValueVectorLarge}
\title{Title
Select singular value vectors from HOSVD (boxplot version)}
\usage{
selectSingularValueVectorLarge(HOSVD, cond, input_all = NULL)
}
\arguments{
\item{HOSVD}{output from HOSVD}

\item{cond}{Labels to select singular value vector number}

\item{input_all}{if list is not null, no interactive mode is
activated but provided values are used.}
}
\value{
Selected singular value vector IDs
}
\description{
Title
Select singular value vectors from HOSVD (boxplot version)
}
\examples{
Z <- PrepareSummarizedExperimentTensor(
sample=matrix(as.character(seq_len(6)),c(3,2)),
feature=as.character(seq_len(10)),
 value=array(runif(10*3*2),c(10,3,2)))
HOSVD <- computeHosvd(Z)
 cond <- list(0,c("A","B","C"),c("A","B"))
 input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(1,1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TDbasedUFE-package.R
\docType{package}
\name{TDbasedUFE-package}
\alias{TDbasedUFE}
\alias{TDbasedUFE-package}
\title{TDbasedUFE: Tensor Decomposition Based Unsupervised Feature Extraction}
\description{
This is a comprehensive package to perform Tensor decomposition based unsupervised feature extraction. It can perform unsupervised feature extraction. It uses tensor decomposition. It is applicable to gene expression, DNA methylation, and histone modification etc. It can perform multiomics analysis. It is also potentially applicable to single cell omics data sets.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/tagtag/TDbasedUFE}
  \item Report bugs at \url{https://github.com/tagtag/TDbasedUFE/issues}
}

}
\author{
\strong{Maintainer}: Y-h. Taguchi \email{tag@granular.com} (\href{https://orcid.org/0000-0003-0867-8986}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectFeature.R
\name{selectFeature}
\alias{selectFeature}
\title{Title
Select features}
\usage{
selectFeature(HOSVD, input_all, de = 1e-04, p0 = 0.01, breaks = 100)
}
\arguments{
\item{HOSVD}{output from HOSVD}

\item{input_all}{Selected singular value IDs}

\item{de}{Initial value for optimization of standard deviation}

\item{p0}{Threshold P-value}

\item{breaks}{The number of bins}
}
\value{
List that includes selected features and computed P-value
}
\description{
Title
Select features
}
\examples{
set.seed(2)
require(rTensor)
HOSVD <- hosvd(as.tensor(array(runif(10000*3*3),c(10000,3,3))),c(10,3,3))
input_all <- c(2,2)
index <- selectFeature(HOSVD,input_all,de=0.01,p0=0.01)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeHosvd.R
\name{computeHosvd}
\alias{computeHosvd}
\title{Title
Compute higher order singular value decomposition}
\usage{
computeHosvd(Z, dims = c(10, dim(attr(Z, "value"))[-1]), scale = TRUE)
}
\arguments{
\item{Z}{array that includes omics data}

\item{dims}{dimensions to be computed by HOSVD}

\item{scale}{If value is scaled}
}
\value{
List that includes output from HOSVD
}
\description{
Title
Compute higher order singular value decomposition
}
\examples{
Z <- PrepareSummarizedExperimentTensor(
sample=matrix(as.character(seq_len(6)),c(3,2)),
feature=as.character(seq_len(10)),
value=array(runif(10*3*2),c(10,3,2)))
HOSVD <- computeHosvd(Z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeHosvdSqure.R
\name{computeHosvdSqure}
\alias{computeHosvdSqure}
\title{Title
Compute higher order singular value decomposition from the
tensor generated from squared matrix}
\usage{
computeHosvdSqure(
  Z,
  dims = unlist(lapply(dim(attr(Z, "value")), function(x) {
     min(10, x)
 })),
  scale = TRUE
)
}
\arguments{
\item{Z}{A tensor including sample names, feature values,
associated with featureRange and sample properties}

\item{dims}{dimensions to be computed by HOSVD}

\item{scale}{If value is scaled}
}
\value{
List that includes output from HOSVD
}
\description{
Title
Compute higher order singular value decomposition from the
tensor generated from squared matrix
}
\examples{
omics1 <- matrix(runif(100),10)
dimnames(omics1) <- list(seq_len(10),seq_len(10))
omics2 <- matrix(runif(100),10)
dimnames(omics2) <- dimnames(omics1)
Multi <- list(omics1,omics2)
Z <- PrepareSummarizedExperimentTensorSquare(
            sample=matrix(colnames(omics1),1),
            feature=list(omics1=rownames(omics1),
            omics2=rownames(omics2)),
            value=convertSquare(Multi),
            sampleData=list(NA))
            HOSVD <- computeHosvdSqure(Z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectSingularValueVectorSmall.R
\name{selectSingularValueVectorSmall}
\alias{selectSingularValueVectorSmall}
\title{Title
Select singular value vectors from HOSVD}
\usage{
selectSingularValueVectorSmall(HOSVD, input_all = NULL)
}
\arguments{
\item{HOSVD}{output from HOSVD}

\item{input_all}{if ist is no null, no interactive mode is
activated but provided values are used.}
}
\value{
Selected singular value vector IDs
}
\description{
Title
Select singular value vectors from HOSVD
}
\examples{
Z <- PrepareSummarizedExperimentTensor(
sample=matrix(as.character(seq_len(6)),c(3,2)),
feature=as.character(seq_len(10)),
 value=array(runif(10*3*2),c(10,3,2)))
 HOSVD <- computeHosvd(Z)
input_all <- selectSingularValueVectorSmall(HOSVD,input_all=c(1,1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectFeatureSquare.R
\name{selectFeatureSquare}
\alias{selectFeatureSquare}
\title{Title
Select features (for tensor generated from squared matrix)}
\usage{
selectFeatureSquare(
  HOSVD,
  input_all,
  Multi,
  de = rep(1e-04, dim(HOSVD$U[[3]])[2]),
  p0 = 0.01,
  breaks = 100,
  interact = TRUE
)
}
\arguments{
\item{HOSVD}{output from HOSVD applied to tensor generated
from squared matrix}

\item{input_all}{Selected singular value vector IDs}

\item{Multi}{Multiomics data}

\item{de}{Initial value for optimization of standard deviation}

\item{p0}{Threshold P-value}

\item{breaks}{The number of bins}

\item{interact}{if interact mode or not}
}
\value{
List that includes selected features and computed P-value
}
\description{
Title
Select features (for tensor generated from squared matrix)
}
\examples{
omics1 <- matrix(runif(100000),ncol=10)
dimnames(omics1) <- list(seq_len(10000),seq_len(10))
omics2 <- matrix(runif(100000),ncol=10)
dimnames(omics2) <- dimnames(omics1)
Multi <- list(omics1,omics2)
Z <- PrepareSummarizedExperimentTensorSquare(
sample=matrix(colnames(omics1),1),
feature=list(omics1=rownames(omics1),
omics2=rownames(omics2)),
value=convertSquare(Multi),
sampleData=list(NA))
HOSVD <- computeHosvdSqure(Z)
cond <- list(0,rep(seq_len(2),each=5),c("A","B"))
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(1,1))
index <- selectFeatureSquare(HOSVD,input_all,Multi,de=c(0.1,0.1),
interact=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableFeaturesSquare.R
\name{tableFeaturesSquare}
\alias{tableFeaturesSquare}
\title{Title
Show selected features as Table (for Squared one)}
\usage{
tableFeaturesSquare(Z, index, id)
}
\arguments{
\item{Z}{Tensor of features}

\item{index}{List that includes selected features and P-values}

\item{id}{feature to be shown}
}
\value{
Table list of selected features
}
\description{
Title
Show selected features as Table (for Squared one)
}
\examples{
omics1 <- matrix(runif(100000),ncol=10)
dimnames(omics1) <- list(seq_len(10000),seq_len(10))
omics2 <- matrix(runif(100000),ncol=10)
dimnames(omics2) <- dimnames(omics1)
Multi <- list(omics1,omics2)
Z <- PrepareSummarizedExperimentTensorSquare(
sample=matrix(colnames(omics1),1),
feature=list(omics1=rownames(omics1),
omics2=rownames(omics2)),
value=convertSquare(Multi),
sampleData=list(NA))
HOSVD <- computeHosvdSqure(Z)
cond <- list(0,rep(seq_len(2),each=5),c("A","B"))
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(1,1))
index <- selectFeatureSquare(HOSVD,input_all,Multi,de=c(0.1,0.1),
interact=FALSE)
index[[1]]$index[1:100]<-TRUE
index[[1]]$p.value[1:100] <- 1e-3
tableFeaturesSquare(Z,index,1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrepareSummarizedExperimentTensor.R
\name{PrepareSummarizedExperimentTensor}
\alias{PrepareSummarizedExperimentTensor}
\title{Title
Generate feature values formatted as a tensor format}
\usage{
PrepareSummarizedExperimentTensor(
  sample,
  feature,
  value,
  featureRange = GRanges(NULL),
  sampleData = list(NULL)
)
}
\arguments{
\item{sample}{Sample names}

\item{feature}{Feature id names}

\item{value}{Feature values}

\item{featureRange}{Genomic coordinate attributed to feature id (if any)}

\item{sampleData}{Sample property (labels etc)}
}
\value{
A tensor including sample names, feature id,
feature values, associated with featureRange and sample properties
}
\description{
Title
Generate feature values formatted as a tensor format
}
\examples{
require(GenomicRanges)
Z <- PrepareSummarizedExperimentTensor(
sample=matrix(as.character(seq_len(6)),c(3,2)),
feature=as.character(seq_len(10)),
 value=array(runif(10*3*2),c(10,3,2)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrepareSummarizedExperimentTensorSquare.R
\name{PrepareSummarizedExperimentTensorSquare}
\alias{PrepareSummarizedExperimentTensorSquare}
\title{Title
Generate feature values formatted as a tensor format from Squared matrix}
\usage{
PrepareSummarizedExperimentTensorSquare(
  sample = list(NULL),
  feature,
  value,
  featureRange = GRanges(NULL),
  sampleData = list(NULL)
)
}
\arguments{
\item{sample}{Sample names}

\item{feature}{Feature id names}

\item{value}{Squared Feature values}

\item{featureRange}{Genomic coordinate attributed to feature id (if any)}

\item{sampleData}{Sample property (labels etc)}
}
\value{
A tensor including sample names, feature values,
associated with featureRange and sample properties
}
\description{
Title
Generate feature values formatted as a tensor format from Squared matrix
}
\examples{
omics1 <- matrix(runif(100),10)
dimnames(omics1) <- list(seq_len(10),seq_len(10))
omics2 <- matrix(runif(100),10)
dimnames(omics2) <- dimnames(omics1)
Multi <- list(omics1,omics2)
Z <- PrepareSummarizedExperimentTensorSquare(
   sample=matrix(colnames(omics1),1),
   feature=list(omics1=rownames(omics1),
   omics2=rownames(omics2)),
   value=convertSquare(Multi),
   sampleData=list(NA))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convertSquare.R
\name{convertSquare}
\alias{convertSquare}
\title{Generate squared tensor from multiomics data}
\usage{
convertSquare(Multi)
}
\arguments{
\item{Multi}{A list that include multiomics data}
}
\value{
A tensor computed from multiomics data
}
\description{
Generate squared tensor from multiomics data
}
\examples{
omics1 <- matrix(runif(100),10)
dimnames(omics1) <- list(seq_len(10),seq_len(10))
omics2 <- matrix(runif(100),10)
dimnames(omics2) <- dimnames(omics1)
Multi <- list(omics1,omics2)
Z <- convertSquare(Multi)
}
